﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using BPA.Message;
using BPA.Message.Enum;
using BPASmartClient.Helper;
using BPASmartClient.Model;
using Microsoft.Toolkit.Mvvm.ComponentModel;

namespace BPASmartClient.ViewModel
{
    public class OrderStatusViewModel : ObservableObject
    {
        public OrderStatusViewModel()
        {

        }

        /// <summary>
        /// 订单状态列表
        /// </summary>
        public static ObservableCollection<OrderData> orderStatusLists
        {
            get { return Json<KeepDataBase>.Data.orderLists; }
            set { Json<KeepDataBase>.Data.orderLists = value; }
        }

        /// <summary>
        /// 等待取餐列表
        /// </summary>
        public static ObservableCollection<OrderData> WaitTakeMeal { get; set; } = new ObservableCollection<OrderData>();

        /// <summary>
        /// MQTT 大屏取餐通知委托
        /// </summary>
        public void MqttReceive()
        {
            //清除订单数据
            ActionManage.GetInstance.Register(new Action(() =>
            {
                Application.Current.Dispatcher.Invoke(() =>
                {
                    orderStatusLists.Clear();
                    WaitTakeMeal.Clear();
                });
            }), "ClearOrders");

            ActionManage.GetInstance.Register(new Action<object>((o) =>
            {
                if (o is MorkOrderPush morkOrderpush)
                {
                    System.Windows.Application.Current?.Dispatcher.BeginInvoke((Action)delegate
                    {
                        //var result = orderStatusLists.FirstOrDefault(p => p.OrderPush.SuborderId == morkOrderpush.SuborderId);
                        //if (result == null)
                        //{
                        //    orderStatusLists.Add(new OrderData()
                        //    {
                        //        OrderPush = morkOrderpush,
                        //        OrderStatus = ORDER_STATUS.WAIT,
                        //        IsSelected = true,
                        //        StartDate = DateTime.Now.ToString("HH:mm:ss")
                        //    });
                        //}
                    });
                }
            }), "AddOrder");

            ActionManage.GetInstance.Register(new Action<object>((o) =>
            {
                if (o is OrderStatusChange orderStatusChange)
                {
                    Application.Current?.Dispatcher.BeginInvoke((Action)delegate
                    {
                        int index = Array.FindIndex(orderStatusLists.ToArray(), p => p.morkOrderPushes.FirstOrDefault(s => s.OrderPush.SuborderId == orderStatusChange.SuborderId) != null);
                        int index2 = -1;
                        if (index >= 0 && index < orderStatusLists.Count)
                        {
                            index2 = Array.FindIndex(orderStatusLists.ElementAt(index).morkOrderPushes.ToArray(), p => p.OrderPush.SuborderId == orderStatusChange.SuborderId);
                        }
                        if (index2 < 0) return;

                        switch (orderStatusChange.CookingStatus)
                        {
                            case ORDER_STATUS.COOKING:
                                if (index >= 0 && index < orderStatusLists.Count)
                                    orderStatusLists.ElementAt(index).morkOrderPushes.ElementAt(index2).OrderStatus = orderStatusChange.CookingStatus;
                                break;
                            case ORDER_STATUS.COMPLETED_COOK:
                                if (index >= 0 && index < orderStatusLists.Count)
                                {
                                    Application.Current.Dispatcher.BeginInvoke((Action)delegate
                                    {
                                        orderStatusLists.ElementAt(index).morkOrderPushes.ElementAt(index2).OrderStatus = orderStatusChange.CookingStatus;
                                        orderStatusLists.ElementAt(index).morkOrderPushes.ElementAt(index2).EndDate = DateTime.Now.ToString("HH:mm:ss");
                                        TimeSpan timeSpan = DateTime.Now.Subtract(Convert.ToDateTime(orderStatusLists.ElementAt(index).morkOrderPushes.ElementAt(index2).StartDate));
                                        orderStatusLists.ElementAt(index).morkOrderPushes.ElementAt(index2).CompleteDate = $"{timeSpan.TotalSeconds.ToString("0.00")} S";
                                        WaitTakeMeal.Insert(0, orderStatusLists.ElementAt(index));
                                        orderStatusLists.RemoveAt(index);
                                    });
                                }

                                break;
                            case ORDER_STATUS.COMPLETED_TAKE:
                                var re = WaitTakeMeal.FirstOrDefault(p => p.morkOrderPushes.FirstOrDefault(s => s.OrderPush.SuborderId == orderStatusChange.SuborderId) != null);
                                if (re != null)
                                {
                                    Application.Current.Dispatcher.BeginInvoke((Action)delegate
                                    {
                                        WaitTakeMeal.Remove(re);
                                        Json<KeepDataBase>.Data.orderLists.Remove(re);
                                    });
                                }
                                break;
                            default:
                                break;
                        }
                    });

                }
            }), "OrderStatusChange");
        }


    }
}
